#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'dictionaries/class/set_search_for_this_word.rb'
# =========================================================================== #
module Dictionaries

class AskWordFromDictionary

  # ========================================================================= #
  # === set_search_for_this_word                                   (menu tag)
  #
  # This method will also have the menu-functionality of the class.
  # ========================================================================= #
  def set_search_for_this_word(i = :rand)
    i = i.join(' ').strip if i.is_a? Array # This will also handle empty ARGV passed to us.
    i = :rand if i.nil?
    case i
    when :rand, 'rand' # We want a random word here.
      i = return_random_word if @words
    end
    # ======================================================================= #
    # We want a String past this point.
    # ======================================================================= #
    i = i.to_s.dup
    # ======================================================================= #
    # We can not downcase the input since we also try to have it match
    # towards regular english letters. It is a dictionary after all.
    # ======================================================================= #
    # === Chop off last char if it is a question mark.
    # ======================================================================= #
    i.chop! if i.end_with? '?'
    case i # case tag
    # ======================================================================= #
    # === askeng --delay?
    # ======================================================================= #
    when /^-?-?delay\??$/i
      feedback_delay
    # ======================================================================= #
    # === askeng --openall
    # ======================================================================= #
    when 'OPEN',
         'OPE',
         'OP',
         'O',
         'EDIT',
         'OPEN_ALL',
         /open/,
         'OPENALL',
         '--openall'
      open_this_file_here(true) # true for "be verbose".
      open_yaml_file_in_main_editor
      exit
    # ======================================================================= #
    # === askeng --n-entries
    # ======================================================================= #
    when 'TOTAL',
         'KEYS',
         'KEYSTATS',
         '--n-entries',
         '--nentries',
         'keys?',
         'nkeys',
         /-?-?nwords\??/
      feedback_total_amount_of_keys
    # ======================================================================= #
    # === askeng --stats
    # ======================================================================= #  
    when '--stats','--statistics','STATS','STATISTICS','STATS?','--stats?'
      feedback_statistics; exit
    # ======================================================================= #
    # === askeng --help
    # ======================================================================= #
    when 'HELP','--help','show_help','--HELP'
      feedback_help_options # This will also exit.
    # ======================================================================= #
    # === askeng --every_word
    # ======================================================================= #
    when 'SHOW',
         '--show-every-word',
         '--every_word'
      show_every_word
      exit
    # ======================================================================= #
    # === askeng --generate-tab
    # ======================================================================= #
    when 'GENERATE',
         'TAB',
         'GENERATE_COMPLETION',
         'TABCOMPLETION',
         'COMPLETION',
         '--generate-tab'
      generate_tab_completion
      exit
    # ======================================================================= #
    # === askeng --open-in-browser
    # ======================================================================= #
    when 'OPEN_BROWSER','--open-in-browser'
      open_in_browser
    # ======================================================================= #
    # === askeng --solved
    # ======================================================================= #
    when 'SOLVED',/-?-?solved/
      set_last_word_solved
    # ======================================================================= #
    # === askeng --yaml-file?
    # ======================================================================= #
    when 'FILE?',
         'YAML_FILE?','--yaml-file?'
      feedback_yaml_file
      exit
    # ======================================================================= #
    # === askeng --open-dictionary
    # ======================================================================= #
    when 'OPEN_DICTIONARY','OPEN_DICT','--open-dictionary'
      open_main_yaml_file_in_main_editor
      exit
    # ======================================================================= #
    # === askeng --repeat
    # ======================================================================= #
    when 'REPEAT','--repeat'
      _ = STORE_LAST_QUESTION_ASKED_HERE
      if File.exist? _
        i = File.read(_)
      else
        opnn; e 'The file `'+sfile(_)+'` was not found, thus we can'
        opnn; e 'not read in anything.'
        exit
      end
    # ======================================================================= #
    # === askeng --url
    # ======================================================================= #
    when '--url','URL'
      feedback_url
    # ======================================================================= #
    # === askeng --random
    # ======================================================================= #
    when /^-?-?random$/i,'RAND','rand' # Pass through here.
      i = return_random_word if available_keys?
    # else tag
    else # Handle assignment cases next.
      if i.include?('=') and i.downcase.include? 'delay'
        new_delay = i.split('=').last.strip
        e 'It seems as if you may want to assign a new delay.'
        e 'We will thus set the last line of this file here'
        e 'with the new delay of:'
        e '  '+sfancy(new_delay)
        what = File.readlines(Dictionaries::AskEnglishWord.main_file?)
        what[-1,1] = new_delay # Set the new entry here.
        SaveFile.write_what_into(what.join, Dictionaries::AskEnglishWord.main_file?)
        exit
      end
    end
    i = i.to_s.dup
    i = i.downcase if SHALL_WE_DOWNCASE
    if i.size == 1 # We assume that the user wants to get a category.
      i = find_all_matches_for(i).sample
    end
    @search_for_this_word = i # We will search for this word here.
  end

end; end