#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'dictionaries/commandline/parse_commandline.rb'
# =========================================================================== #
module Dictionaries

  require 'dictionaries/toplevel_methods/show_help.rb'
  require 'dictionaries/class/ask_word_from_dictionary.rb'
  require 'dictionaries/statistics/statistics.rb'

  # ========================================================================= #
  # === Dictionaries.start_gtk_component
  #
  # This module-method will start the GTK component.
  # ========================================================================= #
  def self.start_gtk_component
    require 'dictionaries/gui/gtk3/dictionary/dictionary.rb'
    Dictionaries::GUI::Gtk::Dictionary.run
  end

  # ========================================================================= #
  # === Dictionaries.parse_commandline
  # ========================================================================= #
  def self.parse_commandline(
      i = ARGV
    )
    if i.is_a? Array
      i.each {|entry| parse_commandline(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === dictionaries --stats
      #
      # This entry point allows the user to show some statistics about
      # this project.
      # ===================================================================== #
      when /^-?-?stats\??$/i,
           /^-?-?statistics\??$/i
        Dictionaries::Statistics.report
      # ===================================================================== #
      # === dictionaries --read=www.nytimes.com.html
      # ===================================================================== #
      when /^-?-?read=(.+)$/i
        _ = $1.to_s.dup
        require 'dictionaries/toplevel_methods/misc.rb'
        if File.exist? _
          e 'Finding unique words ... this may take a while. Please be'
          e 'patient.'
          unique_words = ::Dictionaries.return_unique_words_from_this_file(_, :default, :remove_HTML_tags).sort
          pp unique_words
          e 'n entries: '+unique_words.size.to_s
        else
          e 'No file exists at '+sfile(_)+'.'
        end
      # ===================================================================== #
      # === dictionaries --gui
      # ===================================================================== #
      when /^-?-?gui$/i,
           /^-?-?gtk$/i,
           /^-?-?start(-|_)?gtk$/i
        start_gtk_component
      # ===================================================================== #
      # === dictionaries --sinatra
      # ===================================================================== #
      when /^-?-?sinatra$/i
        require 'dictionaries/sinatra/app.rb'
        Dictionaries.start_sinatra_interface
      # ===================================================================== #
      # === dictionaries --n-words?
      # ===================================================================== #
      when /^-?-?n(_|-)?words\??$/i
        e "#{Colours.sfancy(Dictionaries.n_entries?)} english-to-german "\
          "translations are presently registered in this project."
      # ===================================================================== #
      # === dictionaries --help
      # ===================================================================== #
      when /help/
        show_help
      end
    end
  end

end