#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'dictionaries/constants.rb'
# =========================================================================== #
module Dictionaries

  require 'dictionaries/project/project.rb'
  require 'dictionaries/version/version.rb'

  # ========================================================================= #
  # === Dictionaries::N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === Dictionaries::URL_FOR_DICT_LEO
  # ========================================================================= #
  URL_FOR_DICT_LEO = 'http://dict.leo.org/'

  # ========================================================================= #
  # === URL_FOR_ITALIAN_DICTIONARY
  # ========================================================================= #
  URL_FOR_ITALIAN_DICTIONARY = 'http://www.wordreference.com/iten/'

  # ========================================================================= #
  # === LPAD
  # ========================================================================= #
  LPAD = '  '

  # ========================================================================= #
  # === RUBY_SRC
  # ========================================================================= #
  if ENV['RUBY_SRC']
    RUBY_SRC = ENV['RUBY_SRC'].to_s.dup+'/'
  else # else we just hardcode it anyway.
    RUBY_SRC = '/home/x/programming/ruby/src/'
  end

  # ========================================================================= #
  # === MY_DICTIONARIES
  #
  # This constant is only valid for my own system.
  # ========================================================================= #
  MY_DICTIONARIES = "#{RUBY_SRC}dictionaries/lib/dictionaries/yaml/"

  # ========================================================================= #
  # === SCIENCE_DIR
  # ========================================================================= #
  SCIENCE_DIR = ENV['SCIENCE'].to_s+'/'

  # ========================================================================= #
  # === DEPOT_INFORMATION_DIR
  # ========================================================================= #
  DEPOT_INFORMATION_DIR = '/Depot/Information/'

  # ========================================================================= #
  # === DICTIONARIES_DIR
  #
  # This constant will point at a path such as this one here:
  #
  #   /Programs/Ruby/2.6.3/lib/ruby/site_ruby/2.6.0/dictionaries/yaml/
  #
  # ========================================================================= #
  DICTIONARIES_DIR = Dictionaries.dictionary_directory?

  # ========================================================================= #
  # === ENGLISH_WORDS
  #
  # This constant is no longer that important because we can now
  # automatically infer the name of the .yml file.
  # ========================================================================= #
  ENGLISH_WORDS = DICTIONARIES_DIR+'english.yml'
  ENGLISH_YAML_FILE = ENGLISH_WORDS  # === ENGLISH_YAML_FILE
  FILE_ENGLISH_WORDS = ENGLISH_WORDS # === FILE_ENGLISH_WORDS
  FILE_ENGLISH_DICTIONARY = ENGLISH_YAML_FILE

  # ========================================================================= #
  # === Dictionaries.file_english
  # ========================================================================= #
  def self.file_english
    FILE_ENGLISH_DICTIONARY
  end

  # ========================================================================= #
  # === ITALIAN_WORDS
  #
  # This constant is no longer that important because we can now
  # automatically infer the name of the .yml file.
  # ========================================================================= #
  ITALIAN_WORDS = DICTIONARIES_DIR+'italian.yml'
  ITALIAN_YAML_FILE = ITALIAN_WORDS
  FILE_ITALIAN_DICTIONARY = ITALIAN_YAML_FILE

  # ========================================================================= #
  # === STORE_LINE_NUMBER_HERE
  # ========================================================================= #
  STORE_LINE_NUMBER_HERE = DEPOT_INFORMATION_DIR+'line_number_of_the_last_word'

  # ========================================================================= #
  # === STORE_LAST_ENGLISH_QUESTION_ASKED_HERE
  # ========================================================================= #
  STORE_LAST_ENGLISH_QUESTION_ASKED_HERE =
    DEPOT_INFORMATION_DIR+'last_english_question_asked' # cat $MY_TEMP/last_english_question_asked

  # ========================================================================= #
  # === STORE_LAST_ITALIAN_QUESTION_ASKED_HERE
  # ========================================================================= #
  STORE_LAST_ITALIAN_QUESTION_ASKED_HERE =
    DEPOT_INFORMATION_DIR+'last_italian_question_asked' # cat $MY_TEMP/last_italian_question_asked

  # ========================================================================= #
  # === SHALL_WE_DOWNCASE
  # ========================================================================= #
  SHALL_WE_DOWNCASE = true

  # ========================================================================= #
  # === USE_THIS_ENCODING
  #
  # This is the main encoding to use.
  # ========================================================================= #
  USE_THIS_ENCODING = 'ISO-8859-1'
    MAIN_ENCODING = USE_THIS_ENCODING

  # ========================================================================= #
  # === DEFAULT_DELAY
  #
  # Specify how long to wait before revealing the translated word.
  # ========================================================================= #
  DEFAULT_DELAY = 1.6

end