#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Dictionaries::Sinatra
# =========================================================================== #
# require 'dictionaries/sinatra/app.rb'
# =========================================================================== #
begin

require 'sinatra/base.rb'

module Dictionaries

class Sinatra < ::Sinatra::Base

  require 'dictionaries/class/ask_word_from_dictionary.rb'
  require 'dictionaries/toplevel_methods/is_on_roebe.rb'
  require 'dictionaries/sinatra/english_to_german.rb'

  include ::Colours
  include ::Colours::E
  begin
    require 'html_tags'
    include HtmlTags::BaseModule
  rescue LoadError; end

  # ========================================================================= #
  # === USE_THIS_PORT
  # ========================================================================= #
  USE_THIS_PORT = '5252'

  set :port, USE_THIS_PORT

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i = ARGV
    )
    super()
    reset
    consider_opening_the_page_in_the_browser(i)
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === consider_opening_the_page_in_the_browser
  # ========================================================================= #
  def consider_opening_the_page_in_the_browser(
      shall_we_open_the_page_in_the_browser = ::Dictionaries.is_on_roebe?
    )
    case shall_we_open_the_page_in_the_browser
    # ======================================================================= #
    # === :do_not_open_in_the_browser
    # ======================================================================= #
    when :do_not_open_in_the_browser,
         :dont_connect,
         :dont
      shall_we_open_the_page_in_the_browser = false
    end
    if shall_we_open_the_page_in_the_browser
      begin
        require 'open'
      rescue LoadError; end
      # ===================================================================== #
      # Tell us which port will be used:
      # ===================================================================== #
      target = "http://localhost:#{USE_THIS_PORT}/"
      e sfancy(target)
      Thread.new {
        sleep 0.8
        Open.in_browser(target)
      }
    end
  end

  # ========================================================================= #
  # === / Entry point
  #
  # This is the root of our web-application.
  # ========================================================================= #
  get('/'){
    html_header_default_title_and_start_of_the_body_tag+
    p('The following translation-tables are available:')+
    HtmlTags.a('/english_to_german', css_style: 'margin-left: 2em')+
    br+
    HtmlTags.a('/german_to_english', css_style: 'margin-left: 2em')+
    br
  }

  # ========================================================================= #
  # === not_found
  # ========================================================================= #
  not_found {
    p('This particular <b>API</b> was not found.')
  }

end

# =========================================================================== #
# === Dictionaries.start_sinatra_interface
#
# This method can be used to start the sinatra interface.
# =========================================================================== #
def self.start_sinatra_interface
  e 'Trying to start the sinatra-interface of the Dictionaries project.'
  ::Dictionaries::Sinatra.run!
end

end

rescue LoadError
end

if __FILE__ == $PROGRAM_NAME
  Dictionaries.start_sinatra_interface
end # dictionaries --sinatra