#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'dictionaries/toplevel_methods/main_file.rb'
# =========================================================================== #
module Dictionaries

  # ========================================================================= #
  # === @main_file
  #
  # This module-level instance variable keeps track of the main file in
  # use.
  # ========================================================================= #
  @main_file = nil

  # ========================================================================= #
  # === DICTIONARIES_FILE
  #
  # Add support for the environment variable DICTIONARIES_FILE here.
  # ========================================================================= #
  if ENV['DICTIONARIES_FILE'] and
     File.exist?(ENV['DICTIONARIES_FILE'])
    @main_file = ENV['DICTIONARIES_FILE'].to_s
  end

  # ========================================================================= #
  # === Dictionaries.return_name_from_compound
  #
  # This method will return 'italian' if the input is 'ask_italian_word'.
  # ========================================================================= #
  def self.return_name_from_compound(
      i = Dictionaries.main_file?
    )
    i = i.to_s
    i = File.basename(i).gsub(/.rb/, '').gsub(/.yml/, '')
    if i.include? '_'
      i = i.split('_')[1] # Grab the middle part here.
    end
    i
  end

  # ========================================================================= #
  # === Dictionaries.set_main_file
  #
  # Use this method to designate the main yaml file which should contain
  # the language-specific translations.
  #
  # If the input includes a '/' token and also ends with '.rb' then we
  # assume that it may be in the form of this:
  #
  #   /home/x/programming/ruby/src/dictionaries/lib/dictionaries/ask_italian_word.rb
  #
  # So we will instead use the second part of the last part.
  #
  # Usage example:
  #
  #   Dictionaries.set_main_file(THIS_FILE)
  #
  # ========================================================================= #
  def self.set_main_file(i = :default_file)
    case i
    when :default_file,
         :default
      i = FILE_ENGLISH_WORDS
    end
    if i.include? '/'
      if i.end_with? '.rb' 
        i = File.basename(i).sub(/.rb^/,'')
        if i.include? '_'
          i = Dictionaries.return_name_from_compound(i) # Grab the middle part here.
          i = DICTIONARIES_DIR+i+'.yml'
        end
      end
    end
    @main_file = i
  end

  # ========================================================================= #
  # === Dictionaries.main_file?
  #
  # Query method over the @main_file module-level instance variable.
  # ========================================================================= #
  def self.main_file?
    @main_file
  end; self.instance_eval { alias main_file main_file? } # === Dictionaries.main_file

end